
// ------------------------------------------------------------- //
// Programma di test per display basato su controller NEC D7225G //
// Autore: Emilio (Pietro Giovanni) Ficara                       //
// Data inizio progetto: 16 Marzo 2007                           //
// Microcontrollore: Atmel ATmega48, internal osc, 5V supply     //
// Compilatore: IAR AT90S C-compiler V1.30A/386                  //
// Nota: se il presente file sorgente viene utilizzato per la    //
//       realizzazione di un qualsiasi progetto, si chiede che   //
//       questa intestazione venga riportata interamente.        //
// ------------------------------------------------------------- //

#pragma language=extended       

#include <iom48.h>        
#include <ina90.h>

#define BYTE unsigned char
#define WORD unsigned int
#define LONG unsigned long

#define FW_VERSION  7.316_  // actual firmware release
// Versione 7.316_
// - first release

// I/O definition
#define cs_port PORTB
#define cs_bitmask 0x04
#define si_port PORTB
#define si_bitmask 0x02
#define sck_port PORTB
#define sck_bitmask 0x01
#define res_port PORTC
#define res_bitmask 0x04
#define cd_port PORTC
#define cd_bitmask 0x02
#define busy_port PINC
#define busy_bitmask 0x01

// LCD commands definition
#define DISPLAY_OFF     0x10
#define DISPLAY_ON      0x11
#define WITHOUT_DECODER 0x14
#define WITH_DECODER    0x15
#define BLINK_OFF       0x18
#define BLINK_SLOW      0x1a
#define BLINK_FAST      0x1b
#define CLEAR_MEM       0x20
#define MODE_SET        0x40
#define WRITE_DATA      0xd0
#define LOAD_DP         0xe0

/*
    CHIP pinout and I/O map (28 Pin DIP)
    ------------------------------------

     -------------------------------------------------------------
Pin    10       9      19      18      17      16      15      14
     -------------------------------------------------------------
Port  PB.7    PB.6    PB.5    PB.4    PB.3    PB.2    PB.1    PB.0
Func [XTAL2] [XTAL1]  [SCK]  [MISO]  [OC2A]  [OC1B]  [OC1A]  [ICP1]
Func [TOSC2] [TOSC1]                 [MOSI]   [/SS]          [CLKO]
I/O                                            /cs     si     /sck      

     -------------------------------------------------------------
Pin             1      28      27      26      25      24      23
     -------------------------------------------------------------
Port          PC.6    PC.5    PC.4    PC.3    PC.2    PC.1    PC.0
Func         [/RES]  [ADC5]  [ADC4]  [ADC3]  [ADC2]  [ADC1]  [ADC0]
Func                  [SCL]   [SDA]
I/O                                           /res    c-/d   /busy   

    ---------------------------------------------------------------
Pin    13      12      11       6       5       4       3       2
    ---------------------------------------------------------------
Port  PD.7    PD.6    PD.5    PD.4    PD.3    PD.2    PD.1    PD.0
Func [AIN1]  [AIN0]   [T1]    [T0]   [INT1]  [INT0]   [TXD]   [RXD] 
Func         [OC0A]  [OC0B]  [XCK]   [OC2B]
I/O                                                                  

     -------------------------------------
Pin     7       8     20      21      22
     -------------------------------------
Func [VCC]   [GND]  [AVCC]  [AREF]  [GND]  

*/                      

/*********************************** 
*        Start of variables        * 
***********************************/

#define USER_RAM_LENGTH 6 // bytes of ram used for variables

BYTE ascii[4]; // general purpose array 
BYTE presca; // prescaler for Timer2Overflow
BYTE flags; // general purpose flags
// following variables are NOT cleared at reset

/*********************************** 
*        Interrupts                * 
***********************************/

interrupt [TIMER2_OVF_vect] void TIMER2_OVF_interrupt(void) // tick = 4.096 mS @ 8 MHz 
{  
    if(++presca >= 122) // if 500 mS counted
        {
        presca = 0; // reset prescaler
        flags |= 1; // set 500mS event
        }
}

/*********************************** 
*        Main Program              * 
***********************************/

void uSdelay(BYTE t) // Short delay
{
    for( ; t > 0; --t);
}

void Siout(BYTE d) // Serial output (1 byte)
{
BYTE c;

    while((busy_port & busy_bitmask) == 0); // wait until not busy
    for(c = 0x80; c > 0; c >>= 1) // for all the bits, starting from MSB
        {
        sck_port &= ~sck_bitmask; // set /sck line LOW
        if(d & c) // if bit is '1'
            si_port |= si_bitmask; // set si line HIGH
        else // if bit is '0'
            si_port &= ~si_bitmask; // set si line LOW
        uSdelay(10); // short delay
        sck_port |= sck_bitmask; // set /sck line HIGH
        uSdelay(10); // short delay
        }
}

void Startx(BYTE mode) // Start sending data, mode = c-/d
{
    cs_port &= ~cs_bitmask; // set cs to LOW        
    if(mode) // if mode = command
        cd_port |= cd_bitmask; // set cd to HIGH
    else
        cd_port &= ~cd_bitmask; // set cd to LOW
}
 
void Stoptx(void) // Stop (end) transmission
{
    while((busy_port & busy_bitmask) == 0); // wait until not busy
    cs_port |= cs_bitmask; // set cs to HIGH        
}

void SetDataPointer(BYTE pos) // set data pointer
{
    Startx(1); // start command transmission
    Siout(LOAD_DP | pos); // load data pointer with pos
    Stoptx(); // stop transmission
}

void WriteData(BYTE dat) // write 8 bits data at dp: dp+1:
{
    Startx(0); // start data transmission
    Siout(dat); // load data at dp position
    Stoptx(); // stop transmission
}

void LcdInit(void) // Lcd initialization
{
    res_port &= ~res_bitmask; // put res to LOW
    uSdelay(250); // delay
    res_port |= res_bitmask; // put res to HIGH
    uSdelay(250); // delay
    Startx(1); // start command transmission
    Siout(MODE_SET | 0x00 | 0x02);
    Siout(WITHOUT_DECODER);
    Siout(CLEAR_MEM);
    Siout(DISPLAY_ON);
    Stoptx();
}

void C_task main(void)
{ 
BYTE c, d;

    CLKPR = 0x80; // enable writing to CLKPR register
    CLKPR = 0; // clock frequency = Fosc/1 (8 MHz)

    PORTB = 0xff;   // 1111.1111
    DDRB = 0x07;    // 0000.0111

    PORTC = 0xff;   // 1111.1111
    DDRC = 0x06;    // 0000.0110

    PORTD = 0xff;   // 1111.1111
    DDRD = 0x00;    // 0000.0000

    TCCR2B = 0x05; // T2 runs at clk/128
    TIMSK2 = 0x01; // T2 ovfl interrupt enabled

    for(c = 0; c < USER_RAM_LENGTH; c++) // init all variables
        ascii[c] = 0;

    c = MCUSR; // read MCU reset status
    MCUSR = 0;

    _WDR(); // reset watchdog
    WDTCSR |= 0x18; // enable write
    WDTCSR = 0x0c; // enable watchdog at 250 mS

    _SEI(); // enable interrupts
    
    LcdInit(); // lcd initialize
    for(c = 0, d = 0; ;)  
        {    
        _WDR(); // reset watchdog
        if(flags & 1) // if 500 mS event
            {
            _CLI();
            flags &= ~1; // reset event flag
            _SEI();
            SetDataPointer(c); // set data pointer (must rewrite on the same address) 
            d = d * 2 + 1; // fill byte 1..3..7..0x0f..0x1f..0x3f..0x7f..0xff
            WriteData(d);
            if(d == 0xff) // if filled
                {
                d = 0; // rest filler
                c += 2; // move to next address (2 nibbles)
                if(c >= 28) // 28, not 32 cause not all the memory links to LCD segment
                    c = 0; // restart from address 0
                }
            }
        } 
}
